@@ -85,7 +85,7 @@ CMD = "ml"  # The command line tool.
 EXT_MLM = ".mlm"  # Archive filename extension
 EXT_AIPK = ".aipk"  # Backward compatibility
 
-VERSION = "3.7.11"  # DO NOT MODIFY. Managed from ../Makefile.
+VERSION = "3.7.12"  # DO NOT MODIFY. Managed from ../Makefile.
 
 OPTIONS = {
     # Global command line options
@@ -172,7 +172,15 @@ COMMANDS = {
     },
     "install": {
         "description": "install a named model, local model file or URL",
-        "argument": {"model": {}, "-i": {"help": "SSH key path"}},
+        "argument": {
+            "model": {},
+            "-i": {"help": "SSH key path"},
+            "--yes": {
+                "action": "store_true",
+                "help": 'assume "yes" as answer to all prompts',
+            },
+            "-y": {"action": "store_true", "help": 'the same as "--yes"'},
+        },
         "usage": "  install    <model>   install a named model, local model file or URL",
         "func": "install_model",
         "next": ["configure"],
@@ -221,7 +229,13 @@ COMMANDS = {
     },
     "uninstall": {
         "description": "uninstall a model or all models",
-        "argument": {"model": {"nargs": "?"}},
+        "argument": {
+            "model": {"nargs": "?"},
+            "--yes_cache_no": {
+                "action": "store_true",
+                "help": 'assume "yes" to answer the first question and "no" for the second.',
+            },
+        },
         "usage": "  uninstall    [<model>]  uninstall a model or all models",
         "func": "remove_model",
         "next": ["installed", "install"],
